@@ -34,6 +34,19 @@
 
 #define xFixedToDouble(f) pixman_fixed_to_double(f)
 
+bool
+sna_gradient_is_opaque(const PictGradient *gradient)
+{
+	int n;
+
+	for (n = 0; n < gradient->nstops; n++) {
+		 if (gradient->stops[n].color.alpha < 0xff00)
+			 return false;
+	}
+
+	return true;
+}
+
 static int
 sna_gradient_sample_width(PictGradient *gradient)
 {
@@ -219,11 +232,11 @@ sna_render_flush_solid(struct sna *sna)
 	DBG(("sna_render_flush_solid(size=%d)\n", cache->size));
 	assert(cache->dirty);
 	assert(cache->size);
+	assert(cache->size <= 1024);
 
 	kgem_bo_write(&sna->kgem, cache->cache_bo,
 		      cache->color, cache->size*sizeof(uint32_t));
 	cache->dirty = 0;
-	cache->last = 0;
 }
 
 static void
@@ -250,21 +263,24 @@ sna_render_finish_solid(struct sna *sna, bool force)
 		cache->bo[i] = NULL;
 	}
 
-	old = cache->cache_bo;
-
 	DBG(("sna_render_finish_solid reset\n"));
-
+	old = cache->cache_bo;
 	cache->cache_bo = kgem_create_linear(&sna->kgem, sizeof(cache->color), 0);
 	if (cache->cache_bo == NULL) {
 		cache->cache_bo = old;
 		old = NULL;
 	}
 
-	cache->bo[0] = kgem_create_proxy(&sna->kgem, cache->cache_bo,
-					 0, sizeof(uint32_t));
-	cache->bo[0]->pitch = 4;
 	if (force)
-		cache->size = 1;
+		cache->size = 0;
+	if (cache->last < cache->size) {
+		cache->bo[cache->last] = kgem_create_proxy(&sna->kgem, cache->cache_bo,
+							   cache->last*sizeof(uint32_t), sizeof(uint32_t));
+		if (cache->bo[cache->last])
+			cache->bo[cache->last]->pitch = 4;
+		else
+			cache->last = 1024;
+	}
 
 	if (old)
 		kgem_bo_destroy(&sna->kgem, old);
@@ -283,7 +299,38 @@ sna_render_get_solid(struct sna *sna, uint32_t color)
 
 	if (color == 0xffffffff) {
 		DBG(("%s(white)\n", __FUNCTION__));
-		return kgem_bo_reference(cache->bo[0]);
+		return kgem_bo_reference(sna->render.alpha_cache.bo[255+7]);
+	}
+
+	if ((color >> 24) == 0xff) {
+		int v = 0;
+
+		if (((color >> 16) & 0xff) == 0)
+			v |= 0;
+		else if (((color >> 16) & 0xff) == 0xff)
+			v |= 1 << 2;
+		else
+			v = -1;
+
+		if (((color >> 8) & 0xff) == 0)
+			v |= 0;
+		else if (((color >> 8) & 0xff) == 0xff)
+			v |= 1 << 1;
+		else
+			v = -1;
+
+		if (((color >> 0) & 0xff) == 0)
+			v |= 0;
+		else if (((color >> 0) & 0xff) == 0xff)
+			v |= 1 << 0;
+		else
+			v = -1;
+
+		if (v >= 0) {
+			DBG(("%s(primary (%d,%d,%d): %d)\n",
+			     __FUNCTION__, v & 4, v & 2, v & 1, v));
+			return kgem_bo_reference(sna->render.alpha_cache.bo[255+v]);
+		}
 	}
 
 	if (cache->color[cache->last] == color) {
@@ -292,7 +339,7 @@ sna_render_get_solid(struct sna *sna, uint32_t color)
 		return kgem_bo_reference(cache->bo[cache->last]);
 	}
 
-	for (i = 1; i < cache->size; i++) {
+	for (i = 0; i < cache->size; i++) {
 		if (cache->color[i] == color) {
 			if (cache->bo[i] == NULL) {
 				DBG(("sna_render_get_solid(%d) = %x (recreate)\n",
@@ -306,7 +353,7 @@ sna_render_get_solid(struct sna *sna, uint32_t color)
 		}
 	}
 
-	sna_render_finish_solid(sna, i == ARRAY_SIZE(cache->color));
+	sna_render_finish_solid(sna, i == 1024);
 
 	i = cache->size++;
 	cache->color[i] = color;
@@ -326,7 +373,7 @@ done:
 static bool sna_alpha_cache_init(struct sna *sna)
 {
 	struct sna_alpha_cache *cache = &sna->render.alpha_cache;
-	uint32_t color[256];
+	uint32_t color[256 + 7];
 	int i;
 
 	DBG(("%s\n", __FUNCTION__));
@@ -346,6 +393,28 @@ static bool sna_alpha_cache_init(struct sna *sna)
 
 		cache->bo[i]->pitch = 4;
 	}
+
+	/* primary */
+	for (i = 1; i < 8; i++) {
+		int j = 255+i;
+
+		color[j] = 0xff << 24;
+		if (i & 1)
+			color[j] |= 0xff << 0;
+		if (i & 2)
+			color[j] |= 0xff << 8;
+		if (i & 4)
+			color[j] |= 0xff << 16;
+		cache->bo[j] = kgem_create_proxy(&sna->kgem,
+						 cache->cache_bo,
+						 sizeof(uint32_t)*j,
+						 sizeof(uint32_t));
+		if (cache->bo[j] == NULL)
+			return false;
+
+		cache->bo[j]->pitch = 4;
+	}
+
 	return kgem_bo_write(&sna->kgem, cache->cache_bo, color, sizeof(color));
 }
 
@@ -356,24 +425,14 @@ static bool sna_solid_cache_init(struct sna *sna)
 	DBG(("%s\n", __FUNCTION__));
 
 	cache->cache_bo =
-		kgem_create_linear(&sna->kgem, sizeof(cache->color), 0);
+		kgem_create_linear(&sna->kgem, 4096, 0);
 	if (!cache->cache_bo)
 		return false;
 
-	/*
-	 * Initialise [0] with white since it is very common and filling the
-	 * zeroth slot simplifies some of the checks.
-	 */
-	cache->color[0] = 0xffffffff;
-	cache->bo[0] = kgem_create_proxy(&sna->kgem, cache->cache_bo,
-					 0, sizeof(uint32_t));
-	if (cache->bo[0] == NULL)
-		return false;
-
-	cache->bo[0]->pitch = 4;
-	cache->dirty = 1;
-	cache->size = 1;
-	cache->last = 0;
+	cache->last = 1024;
+	cache->color[cache->last] = 0;
+	cache->dirty = 0;
+	cache->size = 0;
 
 	return true;
 }
@@ -382,6 +441,9 @@ bool sna_gradients_create(struct sna *sna)
 {
 	DBG(("%s\n", __FUNCTION__));
 
+	if (unlikely(sna->kgem.wedged))
+		return true;
+
 	if (!sna_alpha_cache_init(sna))
 		return false;
 
@@ -398,11 +460,15 @@ void sna_gradients_close(struct sna *sna)
 	DBG(("%s\n", __FUNCTION__));
 
 	for (i = 0; i < 256; i++) {
-		if (sna->render.alpha_cache.bo[i])
+		if (sna->render.alpha_cache.bo[i]) {
 			kgem_bo_destroy(&sna->kgem, sna->render.alpha_cache.bo[i]);
+			sna->render.alpha_cache.bo[i] = NULL;
+		}
 	}
-	if (sna->render.alpha_cache.cache_bo)
+	if (sna->render.alpha_cache.cache_bo) {
 		kgem_bo_destroy(&sna->kgem, sna->render.alpha_cache.cache_bo);
+		sna->render.alpha_cache.cache_bo = NULL;
+	}
 
 	if (sna->render.solid_cache.cache_bo)
 		kgem_bo_destroy(&sna->kgem, sna->render.solid_cache.cache_bo);
